#!/usr/bin/env python
# -*- coding: utf-8 -*-

from unittest import TestCase, main
from excalibur.loader import ConfigurationLoader
from excalibur.core import PluginsRunner
from excalibur.core import Query


class PluginsTest(TestCase):

    def setUp(self):
        # Params of the deactivate method for uds
        self.query = Query(source="etab1",
                           remote_ip="127.0.0.1",
                           signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
                           arguments={"login": "testzombie1", },
                           ressource="actions",
                           method="action1",
                           request_method="GET"
                           )

        self.query2 = Query(source="etab1",
                            remote_ip="127.0.0.1",
                            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
                            arguments={"login": "testzombie1", },
                            ressource="actions",
                            method="action2",
                            request_method="GET"
                            )

        # Files
        self.plugin_runner = PluginsRunner(
            "./tests/data/acl.yml", "./tests/data/sources.yml", "./tests/data/ressources.yml", "tests.plugins", self.query)

        self.data_ok = {'Plugin1': 'p1ok1', 'Plugin2': 'p2ok1'}
        self.errors_raw = {'Plugin1': {'method': 'action2', 'source': 'etab1', 'arguments': {'login': 'testzombie1'}, 'error': 'Exception', 'ressource': 'actions', 'parameters_index': 0}, 'Plugin2': {
            'method': 'action2', 'source': 'etab1', 'arguments': {'login': 'testzombie1'}, 'error': 'Exception', 'ressource': 'actions', 'parameters_index': 0}}

    def test_run_plugins(self):
        data, errors = self.plugin_runner.run_plugins()

        self.assertEqual(data, self.data_ok)
        self.assertEqual(errors, {})

    def test_run_plugins_error(self):
        plugin_runner = PluginsRunner(
            "./tests/data/acl.yml", "./tests/data/sources.yml", "./tests/data/ressources.yml", "tests.plugins", self.query2)
        data, errors = plugin_runner.run_plugins()

        self.assertEqual(errors, self.errors_raw)
        self.assertEqual(data, {})


class RunnerTest(TestCase):

    def setUp(self):

        self.query = Query(source="etab1",
                           remote_ip="127.0.0.1",
                           signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
                           arguments={"login": "testzombie1", },
                           ressource="actions",
                           method="action1",
                           request_method="GET"
                           )

        self.query2 = Query(source="etab1",
                            remote_ip="127.0.0.1",
                            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
                            arguments={"login": "testzombie1", },
                            ressource="actions",
                            method="action2",
                            request_method="GET"
                            )

        self.plugin_runner = PluginsRunner(
            "./tests/data/acl.yml", "./tests/data/sources.yml", "./tests/data/ressources.yml", "tests.plugins", self.query)

        self.data_ok = {'Plugin1': 'p1ok1', 'Plugin2': 'p2ok1'}
        self.errors_raw = {'Plugin1': {'method': 'action2', 'source': 'etab1', 'arguments': {'login': 'testzombie1'}, 'error': 'Exception', 'ressource': 'actions', 'parameters_index': 0}, 'Plugin2': {
            'method': 'action2', 'source': 'etab1', 'arguments': {'login': 'testzombie1'}, 'error': 'Exception', 'ressource': 'actions', 'parameters_index': 0}}

    def test_runner(self):
        data, errors = self.plugin_runner()
        self.assertEqual(data, self.data_ok)
        self.assertEqual(errors, {})

    def test_runner_errors(self):

        plugin_runner = PluginsRunner(
            "./tests/data/acl.yml", "./tests/data/sources.yml", "./tests/data/ressources.yml", "tests.plugins", self.query2)
        data, errors = plugin_runner()
        self.assertEqual(errors, self.errors_raw)
        self.assertEqual(data, {})


class RunnerWithProjectsTest(TestCase):

    def setUp(self):

        self.query = Query(source="etab1",
                           remote_ip="127.0.0.1",
                           signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
                           arguments={"login": "testzombie1", },
                           ressource="actions",
                           method="action1",
                           request_method="GET",
                           project="project1"
                           )

        self.query2 = Query(source="etab1",
                            remote_ip="127.0.0.1",
                            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
                            arguments={"login": "testzombie1", },
                            ressource="actions",
                            method="action2",
                            request_method="GET",
                            project="project1"
                            )

        self.plugin_runner = PluginsRunner(
            "./tests/data/acl_projects.yml", "./tests/data/sources_projects.yml", "./tests/data/ressources.yml", "tests.plugins", self.query)

        self.data_ok = {'Plugin1': 'p1ok1', 'Plugin2': 'p2ok1'}
        self.errors_raw = {'Plugin1': {'method': 'action2', 'source': 'etab1', 'arguments': {'login': 'testzombie1'}, 'error': 'Exception', 'ressource': 'actions', 'parameters_index': 0}, 'Plugin2': {
            'method': 'action2', 'source': 'etab1', 'arguments': {'login': 'testzombie1'}, 'error': 'Exception', 'ressource': 'actions', 'parameters_index': 0}}

    def test_runner(self):
        data, errors = self.plugin_runner()
        self.assertEqual(data, self.data_ok)
        self.assertEqual(errors, {})

    def test_runner_errors(self):

        plugin_runner = PluginsRunner(
            "./tests/data/acl_projects.yml", "./tests/data/sources_projects.yml", "./tests/data/ressources.yml", "tests.plugins", self.query2)
        data, errors = plugin_runner()
        self.assertEqual(errors, self.errors_raw)
        self.assertEqual(data, {})


if __name__ == '__main__':
    main()



class Plugin1(object):

    def actions_action1(self, parameters, arguments):
        return "p1ok1"

    def actions_action2(self, parameters, arguments):
        raise Exception("error plugin 1 action 2 !")



# -*- coding: utf-8 -*-


class ExcaliburError(Exception):

    """
    base exception for excalibur
    """

    def __init__(self, *args, **kwargs):
        super(ExcaliburError, self).__init__(*args, **kwargs)

    def __str__(self):
        return self.message



class ExcaliburClientError(ExcaliburError):

    """
    base exception for excalibur
    """

    def __init__(self, *args, **kwargs):
        super(ExcaliburClientError, self).__init__(*args, **kwargs)

    def __str__(self):
        return self.message


class ExcaliburInternalError(ExcaliburError):

    """
    base exception for excalibur
    """

    def __init__(self, *args, **kwargs):
        super(ExcaliburInternalError, self).__init__(*args, **kwargs)

    def __str__(self):
        return self.message


class ConfigurationLoaderError(ExcaliburInternalError):

    """
    base exception for configuration loader
    """

    def __init__(self, message, *args, **kwargs):
        super(ConfigurationLoaderError, self).__init__(*args, **kwargs)
        self.message = '%s : %s ' % (self.__class__.__name__, message)

    def __str__(self):
        return self.message


class PluginLoaderError(ExcaliburInternalError):

    """
    base exception for plugin loader
    """

    def __init__(self, message, *args, **kwargs):
        super(PluginLoaderError, self).__init__(*args, **kwargs)
        self.message = '%s : %s' % (self.__class__.__name__, message)

    def __str__(self):
        return self.message


class ArgumentError(ExcaliburClientError):

    """
    check error
    """

    def __init__(self, message, *args, **kwargs):
        super(ArgumentError, self).__init__(*args, **kwargs)
        self.message = '%s : %s' % (self.__class__.__name__, message)

    def __str__(self):
        return self.message


class ArgumentCheckMethodNotFoundError(ExcaliburInternalError):

    """
    check error
    """

    def __init__(self, message, *args, **kwargs):
        super(ArgumentCheckMethodNotFoundError, self).__init__(*args, **kwargs)
        self.message = '%s : %s' % (self.__class__.__name__, message)

    def __str__(self):
        return self.message


class CheckMethodError(ExcaliburInternalError):

    """
    check error
    """

    def __init__(self, message, *args, **kwargs):
        super(CheckMethodError, self).__init__(*args, **kwargs)
        self.message = '%s : %s' % (self.__class__.__name__, message)

    def __str__(self):
        return self.message


class NoACLMatchedError(ExcaliburClientError):

    """
    check error
    """

    def __init__(self, message, *args, **kwargs):
        super(NoACLMatchedError, self).__init__(*args, **kwargs)
        self.message = '%s : %s' % (self.__class__.__name__, message)

    def __str__(self):
        return self.message


class RessourceNotFoundError(ExcaliburClientError):

    """
    check error
    """

    def __init__(self, message, *args, **kwargs):
        super(RessourceNotFoundError, self).__init__(*args, **kwargs)
        self.message = '%s : %s' % (self.__class__.__name__, message)

    def __str__(self):
        return self.message


class MethodNotFoundError(ExcaliburClientError):

    """
    check error
    """

    def __init__(self, message, *args, **kwargs):
        super(MethodNotFoundError, self).__init__(*args, **kwargs)
        self.message = '%s : %s' % (self.__class__.__name__, message)

    def __str__(self):
        return self.message


class HTTPMethodError(ExcaliburClientError):

    """
    check error
    """

    def __init__(self, message, *args, **kwargs):
        super(HTTPMethodError, self).__init__(*args, **kwargs)
        self.message = '%s : %s' % (self.__class__.__name__, message)

    def __str__(self):
        return self.message


class SourceNotFoundError(ExcaliburClientError):

    """
    check error
    """

    def __init__(self, message, *args, **kwargs):
        super(SourceNotFoundError, self).__init__(*args, **kwargs)
        self.message = '%s : %s' % (self.__class__.__name__, message)

    def __str__(self):
        return self.message


class IPNotAuthorizedError(ExcaliburClientError):

    """
    check error
    """

    def __init__(self, message, *args, **kwargs):
        super(IPNotAuthorizedError, self).__init__(*args, **kwargs)
        self.message = '%s : %s' % (self.__class__.__name__, message)

    def __str__(self):
        return self.message


class WrongSignatureError(ExcaliburClientError):

    """
    check error
    """

    def __init__(self, message, *args, **kwargs):
        super(WrongSignatureError, self).__init__(*args, **kwargs)
        self.message = '%s : %s' % (self.__class__.__name__, message)

    def __str__(self):
        return self.message


class DecodeAlgorithmNotFoundError(ExcaliburInternalError):

    """
    check error
    """

    def __init__(self, message, *args, **kwargs):
        super(DecodeAlgorithmNotFoundError, self).__init__(*args, **kwargs)
        self.message = '%s : %s' % (self.__class__.__name__, message)

    def __str__(self):
        return self.message




# -*- coding: utf-8 -*-
"""
Class to load configuration and plugins
"""

from excalibur.exceptions import ExcaliburError, ConfigurationLoaderError, PluginLoaderError
import importlib
import yaml


class ConfigurationLoader(object):

    """
    load config from yaml
    """

    def __init__(self, file_path):
        try:
            with open(file_path, "r") as configuration_file:
                self.__content = yaml.load(configuration_file)
        except Exception:
            raise ConfigurationLoaderError(
                "error with the configuration loader: %s" % file_path)

    @property
    def content(self):
        """
        Return configuration content.
        """
        return self.__content


class PluginLoader(object):

    """
    Create plugin instance
    """

    def __init__(self, plugin_module):
        self.plugin_module = plugin_module

    def get_plugin(self, plugin_name):
        """
        return plugin instance
        """
        try:
            module = importlib.import_module(
                "%s.%s" % (self.plugin_module, plugin_name))
            plugin = getattr(module, plugin_name)
            return plugin()
        except Exception as e:
            raise PluginLoaderError(
                "Plugin %s failed to load: %s" % (plugin_name, e))



# -*- coding: utf-8 -*-
import hashlib
import re

from excalibur.exceptions import ArgumentError, ArgumentCheckMethodNotFoundError, CheckMethodError,\
    NoACLMatchedError, RessourceNotFoundError, MethodNotFoundError, HTTPMethodError, SourceNotFoundError, \
    IPNotAuthorizedError, WrongSignatureError


class Check(object):

    """
    Classe mere pour implementer les Checks.
    """

    def check(self):
        raise NotImplementedError


class CheckArguments(Check):

    """
    Classe verifiant la consistance des arguments.

    Pour implementer un nouveau test, il suffit d'implementer une nouvelle methode qui
    doit se nommer check_nomdutest. Les arguments passes doivent etre la valeur a tester
    et une valeur servant au test. La valeur de retour doit etre un booleen.
    """

    def __init__(self, ressources):
        self.ressources = ressources

    def check(self, arguments, ressource, method):
        errors = {}  # Garde la trace des arguments qui ont echoue aux checks
        for argument_name in arguments:
            # Construction et verification de la batterie de checks
            # (check_list)
            try:
                check_list = self.ressources[ressource][method][
                    "arguments"][argument_name]["checks"]
            except KeyError:
                raise ArgumentError("unexpected argument %s" % argument_name)

            for check in check_list:
                try:
                    check_method_name = "check_" + check.replace(" ", "_")
                    check_method = getattr(self, check_method_name)
                    check_parameter = self.ressources[ressource][method][
                        "arguments"][argument_name]["checks"][check]
                    value_to_check = arguments[argument_name]

                    if not check_method(value_to_check, check_parameter):
                        errors[argument_name] = check
                except AttributeError:
                    raise ArgumentCheckMethodNotFoundError(check_method_name)
                except Exception as e:
                    # Erreur dans la 'check method'
                    raise CheckMethodError(e)

        if errors:
            raise ArgumentError("The check list did not pass", errors)

    def check_min_length(self, argument_value, length):
        return len(argument_value) >= length

    def check_max_length(self, argument_value, length):
        return len(argument_value) <= length

    def check_value_in(self, argument_value, choices):
        return argument_value in choices

    def check_matches_re(self, argument_value, re_string):
        regex = re.compile(re_string)

        return regex.match(argument_value) is not None


class CheckACL(Check):

    """
    Verifie les acces aux ressources et methodes. Les ACL sont definies dans
    le fichier acl.yml.
    """

    def __init__(self, acl):
        self.acl = acl

    def check(self, source, ressource, method, project=None):
        if project:
            if method not in self.acl[project][source][ressource]:
                raise NoACLMatchedError("%s/%s" % (ressource, method))
        else:
            if method not in self.acl[source][ressource]:
                raise NoACLMatchedError("%s/%s" % (ressource, method))


class CheckRequest(Check):

    """
    Verifie la requete realisee sur divers criteres. La verification d'un critere
    est effectuee par l'appel de l'une des methodes definies ci-dessous.

    Les criteres attendus sont definis dans le fichier ressources.yml.
    """

    def __init__(self, ressources):
        self.ressources = ressources

    def check(self, http_method, ressource, method, arguments):
        if ressource not in self.ressources:
            raise RessourceNotFoundError(ressource)
        if method not in self.ressources[ressource]:
            raise MethodNotFoundError(method)
        if http_method != self.ressources[ressource][method]["request method"]:
            raise HTTPMethodError(
                self.ressources[ressource][method]["request method"])

        expected_nb_arguments = len(
            self.ressources[ressource][method]["arguments"])
        received_nb_arguments = len(arguments)

        if expected_nb_arguments != received_nb_arguments:
            raise ArgumentError("Unexpected number of arguments: %i (expected %i)" % (
                received_nb_arguments, expected_nb_arguments))


class CheckSource(Check):

    """
    S'assure que la source est legitime en suivant les donnees presentes
    dans le fichier sources.yml.
    """

    def __init__(self, sources):
        self.sources = sources

    def check(self, source, ip, signature, arguments):
        if source not in self.sources:
            raise SourceNotFoundError("Unknown source %s" % source)

        # Check if IP is authorized
        if ip not in self.sources[source]["ip"]:
            raise IPNotAuthorizedError(ip)

        # Signature check
        arguments_list = sorted(arguments)

        to_hash = self.sources[source]["apikey"]

        for argument in arguments_list:
            to_hash += (argument + arguments[argument])

        signkey = hashlib.sha1(to_hash.encode("utf-8")).hexdigest()

        if signature != signkey:
            raise WrongSignatureError(signature)



# -*- coding: utf-8 -*-
"""
Module qui permet de gerer l'encodage de certains arguments. Comme par exemple,
l'encodage en base64 pour palier au probleme de caracteres speciaux non geres par
les urls.
"""
from excalibur.exceptions import ArgumentError, DecodeAlgorithmNotFoundError


class DecodeArguments(object):

    """
    Classe contenant les differents algorithmes de decodage.

    Pour implementer un nouvel algorithme, une methode suivant la
    regle de nommage suivante decode_nomalgo doit recevoir la
    valeur en argument. Et retourner la valeur decodee.
    """

    def __init__(self, ressources):
        self.ressources = ressources

    def decode(self, ressource, method_name, arguments):
        """
        Pour chacun des arguments passes, regarde s'il doit etre decode.
        Et si c'est le cas, appelle la methode correspondante.
        """
        try:
            for argument_name in arguments:
                if "encoding" in self.ressources[ressource][method_name]["arguments"][argument_name]:
                    algo = self.ressources[ressource][method_name][
                        "arguments"][argument_name]["encoding"]
                    method = getattr(self, "decode_" + algo)
                    arguments[argument_name] = method(arguments[argument_name])
        except KeyError:
            raise ArgumentError(argument_name)
        except AttributeError:
            raise DecodeAlgorithmNotFoundError(algo)

    def decode_base64(self, value):
        """
        Implemente le decodage base64.
        """

        # base64 decoding returns bytestrings
        decoded_value = value.decode("base64").decode("utf-8")
        return decoded_value



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from unittest import TestCase, main
from excalibur.loader import ConfigurationLoader, PluginLoader
from excalibur.exceptions import ConfigurationLoaderError, PluginLoaderError
from tests.plugins.Plugin1 import Plugin1
from tests.plugins.Plugin2 import Plugin2


class ConfigurationLoaderTest(TestCase):

    def setUp(self):
        self.file_path_ok = "./tests/data/sources.yml"
        self.file_path_wrong = "/tmp/doesntexist"
        self.raw = {'etab2': {'ip': ['127.0.0.1'], 'apikey': 'S3CR3T2', 'plugins': {'Plugin1': [{'spore': 'S3CR3T2', 'token': 'S3CR3T2'}], 'Plugin2': [{'spore': 'S3CR3T2', 'token': 'S3CR3T2'}]}}, 'etab1': {
            'ip': ['127.0.0.1'], 'apikey': 'S3CR3T', 'plugins': {'Plugin1': [{'spore': 'S3CR3T', 'token': 'S3CR3T'}], 'Plugin2': [{'spore': 'S3CR3T', 'token': 'S3CR3T'}]}}}
        self.raw_wrong = {'error': 'error'}

    def test_load_content_ok(self):
        c = ConfigurationLoader(self.file_path_ok)
        self.assertEqual(c.content, self.raw)

    def test_load_content_wrong(self):
        c = ConfigurationLoader(self.file_path_ok)
        self.assertNotEqual(c.content, self.raw_wrong)

    def test_load_content_doesntexist(self):
        with self.assertRaises(ConfigurationLoaderError):
            ConfigurationLoader(self.file_path_wrong)


class PluginLoaderTest(TestCase):

    def setUp(self):
        self.plugin_module = "tests.plugins"
        self.plugin1 = "Plugin1"
        self.plugin_doesntexist = "NotExist"
        self.plugin_wrong = "Plugin2"

    def test_load_plugin_ok(self):
        p = PluginLoader(self.plugin_module)
        instance = p.get_plugin(self.plugin1)
        self.assertIsInstance(instance, Plugin1)

    def test_load_plugin_wrong(self):
        p = PluginLoader(self.plugin_module)
        instance = p.get_plugin(self.plugin1)
        self.assertNotIsInstance(instance, Plugin2)

    def test_load_plugin_doesntexist(self):
        with self.assertRaises(PluginLoaderError):
            p = PluginLoader(self.plugin_doesntexist)
            instance = p.get_plugin(self.plugin1)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
class to run all the process
"""

from excalibur.loader import ConfigurationLoader, PluginLoader
from excalibur.check import CheckACL, CheckArguments, CheckRequest, CheckSource
from excalibur.decode import DecodeArguments


class PluginsRunner(object):

    def __init__(self, acl_file, sources_file, ressources_file, plugins_module, query):
        self.__acl = ConfigurationLoader(acl_file).content
        self.__sources = ConfigurationLoader(sources_file).content
        self.__ressources = ConfigurationLoader(ressources_file).content
        self.__plugins_module = plugins_module
        self.__query = query

    @property
    def plugins(self):
        return self.sources[self.__query.source]["plugins"]

    @property
    def acl(self):
        return self.__acl

    @property
    def sources(self):
        if self.__query.project:
            return self.__sources[self.__query.project]["sources"]
        else:
            return self.__sources

    @property
    def ressources(self):
        return self.__ressources

    @property
    def plugins_module(self):
        return self.__plugins_module

    @property
    def query(self):
        return self.__query

    def __call__(self):
        self.check_all()
        data, errors = self.run_plugins()
        return data, errors

    def check_all(self):
        """
        check all yml
        """

        check_source = CheckSource(self.sources)
        check_source.check(self.__query.source, self.__query.remote_ip,
                           self.__query.signature, self.__query.arguments)

        check_acl = CheckACL(self.__acl)
        check_acl.check(self.__query.source, self.__query.ressource,
                        self.__query.method, self.__query.project)

        check_request = CheckRequest(self.__ressources)
        check_request.check(
            self.__query.request_method, self.__query.ressource, self.__query.method, self.__query.arguments)

        decode_arguments = DecodeArguments(self.__ressources)
        decode_arguments.decode(
            self.__query.ressource, self.__query.method, self.__query.arguments)

        check_arguments = CheckArguments(self.__ressources)
        check_arguments.check(
            self.__query.arguments, self.__query.ressource, self.__query.method)

    def run_plugins(self):
        """
        Parcours les plugins et execute la méthode demandée
        """

        data = {}
        errors = {}
        plugin_loader = PluginLoader(self.__plugins_module)

        for plugin_name, parameters_sets in self.plugins.items():
            for parameters_index, parameters in enumerate(parameters_sets):
                plugin = plugin_loader.get_plugin(plugin_name)
                plugin_data = None
                # Name of the function to run
                f_name = "%s_%s" % (
                    self.__query.ressource, self.__query.method)
                if not hasattr(plugin, f_name):
                    continue  # Ressource/method not implemented in plugin
                try:
                    f = getattr(plugin, f_name)
                    plugin_data = f(parameters, self.__query.arguments)
                except Exception as e:
                    errors[plugin_name] = {'source': self.__query.source,
                                           'ressource': self.__query.ressource,
                                           'method': self.__query.method,
                                           'arguments': self.__query.arguments,
                                           'parameters_index': parameters_index,
                                           'error': e.__class__.__name__
                                           }

                if plugin_data is not None:
                    data[plugin_name] = plugin_data

        return data, errors


class Query(object):

    """
    """

    def __init__(self, source, remote_ip, signature, ressource, method, request_method, project=None, arguments=None):
        self.__project = project
        self.__source = source
        self.__remote_ip = remote_ip
        self.__signature = signature
        self.__arguments = arguments if arguments else {}
        self.__ressource = ressource
        self.__method = method
        self.__request_method = request_method

    def __str__(self):
        return dir(self)

    @property
    def project(self):
        return self.__project

    @property
    def source(self):
        return self.__source

    @property
    def remote_ip(self):
        return self.__remote_ip

    @property
    def signature(self):
        return self.__signature

    @property
    def arguments(self):
        return self.__arguments

    @property
    def ressource(self):
        return self.__ressource

    @property
    def method(self):
        return self.__method

    @property
    def request_method(self):
        return self.__request_method



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
from setuptools import find_packages

with open('README.rst') as readme:
    long_description = readme.read()

with open('requirements.txt') as requirements:
    lines = requirements.readlines()
    libraries = [lib for lib in lines if not lib.startswith('-')]
    dependency_links = [link.split()[1] for link in lines if
                        link.startswith('-f')]

setup(
    name='excalibur',
    version='1.0.0',
    author='di-dip-unistra',
    author_email='di-dip@unistra.fr',
    maintainer='di-dip-unistra',
    maintainer_email='di-dip@unistra.fr',
    url='https://github.com/unistra/excalibur',
    license='PSF',
    description='A tool to manage plugins',
    long_description=long_description,
    packages=find_packages(),
    download_url='http://pypi.python.org/pypi/excalibur',
    install_requires=libraries,
    dependency_links=dependency_links,
    keywords=['plugins', 'yaml', 'parser'],
    entry_points={},
    classifiers=(
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3'
    )
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from unittest import TestCase, main
from excalibur.decode import DecodeArguments
from excalibur.loader import ConfigurationLoader
from excalibur.check import CheckSource, CheckACL, CheckRequest, CheckArguments
from excalibur.exceptions import SourceNotFoundError, IPNotAuthorizedError, WrongSignatureError, NoACLMatchedError,\
    RessourceNotFoundError, MethodNotFoundError, HTTPMethodError, ArgumentError, ArgumentCheckMethodNotFoundError
from excalibur.core import PluginsRunner, Query


class CheckTest(TestCase):

    """
    Unit test for check method
    Warning : "Params" of the setUp method need to be valid with
    your yaml files
    """

    def setUp(self):
        # Params of the deactivate method for uds
        self.query = Query(
            source="etab1",
            remote_ip="127.0.0.1",
            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
            arguments={"login": "testzombie1", },
            ressource="actions",
            method="action1",
            request_method="GET"
        )
        # Files
        self.plugin_runner = PluginsRunner(
            "./tests/data/acl.yml", "./tests/data/sources.yml", "./tests/data/ressources.yml", "tests.plugins", self.query)

    """
    Check Source
    """

    def test_check_source(self):
        """ test check sources """
        try:
            check_source = CheckSource(self.plugin_runner.sources)
            check_source.check(self.plugin_runner.query.source, self.plugin_runner.query.remote_ip,
                               self.plugin_runner.query.signature, self.plugin_runner.query.arguments)
        except:
            self.fail("Error check source")

    def test_check_source_not_found(self):
        """ test check sources """
        source = "etabnull"
        with self.assertRaises(SourceNotFoundError):
            check_source = CheckSource(self.plugin_runner.sources)
            check_source.check(source, self.plugin_runner.query.remote_ip,
                               self.plugin_runner.query.signature, self.plugin_runner.query.arguments)

    def test_check_source_ip_not_authorized(self):
        """ test check sources """
        remote_ip = "9.9.9.9"
        with self.assertRaises(IPNotAuthorizedError):
            check_source = CheckSource(self.plugin_runner.sources)
            check_source.check(self.plugin_runner.query.source, remote_ip,
                               self.plugin_runner.query.signature, self.plugin_runner.query.arguments)

    def test_check_source_signature_error(self):
        """ test check sources """
        signature = "ERROR"
        with self.assertRaises(WrongSignatureError):
            check_source = CheckSource(self.plugin_runner.sources)
            check_source.check(self.plugin_runner.query.source, self.plugin_runner.query.remote_ip,
                               signature, self.plugin_runner.query.arguments)

    """
    Check ACL
    """

    def test_check_acl(self):
        """ test check acl """
        try:
            check_acl = CheckACL(self.plugin_runner.acl)
            check_acl.check(self.plugin_runner.query.source,
                            self.plugin_runner.query.ressource, self.plugin_runner.query.method)
        except:
            self.fail("Error check acl")

    def test_check_acl_no_matched(self):
        """ test check acl """
        method = "actionull"
        with self.assertRaises(NoACLMatchedError):
            check_acl = CheckACL(self.plugin_runner.acl)
            check_acl.check(
                self.plugin_runner.query.source, self.plugin_runner.query.ressource, method)

    """
    Check Request
    """

    def test_check_request(self):
        """ test check request """
        try:
            check_request = CheckRequest(self.plugin_runner.ressources)
            check_request.check(self.plugin_runner.query.request_method, self.plugin_runner.query.ressource,
                                self.plugin_runner.query.method, self.plugin_runner.query.arguments)
        except:
            self.fail("error check request")

    def test_check_request_ressource_not_found(self):
        """ test check request """
        ressource = "ressourcenull"
        with self.assertRaises(RessourceNotFoundError):
            check_request = CheckRequest(self.plugin_runner.ressources)
            check_request.check(self.plugin_runner.query.request_method, ressource,
                                self.plugin_runner.query.method, self.plugin_runner.query.arguments)

    def test_check_request_method_not_found(self):
        """ test check request """
        method = "methodnull"
        with self.assertRaises(MethodNotFoundError):
            check_request = CheckRequest(self.plugin_runner.ressources)
            check_request.check(self.plugin_runner.query.request_method,
                                self.plugin_runner.query.ressource, method, self.plugin_runner.query.arguments)

    def test_check_request_method_http_error(self):
        """ test check request """
        request_method = "GETnull"
        with self.assertRaises(HTTPMethodError):
            check_request = CheckRequest(self.plugin_runner.ressources)
            check_request.check(request_method, self.plugin_runner.query.ressource,
                                self.plugin_runner.query.method, self.plugin_runner.query.arguments)

    def test_check_request_args_error(self):
        """ test check request """
        arguments = "argumentsnull"
        with self.assertRaises(ArgumentError):
            check_request = CheckRequest(self.plugin_runner.ressources)
            check_request.check(self.plugin_runner.query.request_method,
                                self.plugin_runner.query.ressource, self.plugin_runner.query.method, arguments)

    """
    Check arguments
    """

    def test_check_arguments(self):
        """ test check arguments """
        try:
            decode_arguments = DecodeArguments(self.plugin_runner.ressources)
            decode_arguments.decode(
                self.plugin_runner.query.ressource, self.plugin_runner.query.method, self.plugin_runner.query.arguments)

            check_arguments = CheckArguments(self.plugin_runner.ressources)
            check_arguments.check(self.plugin_runner.query.arguments,
                                  self.plugin_runner.query.ressource, self.plugin_runner.query.method)
        except:
            self.fail("error check arguments")

    def test_check_arguments_error(self):
        """ test check arguments """
        arguments = {"login": "a", }
        with self.assertRaises(ArgumentError):
            decode_arguments = DecodeArguments(self.plugin_runner.ressources)
            decode_arguments.decode(
                self.plugin_runner.query.ressource, self.plugin_runner.query.method, arguments)

            check_arguments = CheckArguments(self.plugin_runner.ressources)
            check_arguments.check(
                arguments, self.plugin_runner.query.ressource, self.plugin_runner.query.method)

    """
    Check all
    """

    def test_check_all(self):
        try:
            self.plugin_runner.check_all()
        except:
            self.fail("error check all")


if __name__ == '__main__':
    main()
